/** ###################################################################
**     Filename  : DC17.C
**     Project   : DC17
**     Processor : MC56F8006_32_LQFP
**     Version   : Driver 01.13
**     Compiler  : Metrowerks DSP C Compiler
**     Date/Time : 1/18/2009, 1:37 PM
**     Abstract  :
**         Main module.
**         This module contains user's application code.
**     Settings  :
**     Contents  :
**         No public methods
**
**     (c) Copyright UNIS, a.s. 1997-2008
**     UNIS, a.s.
**     Jundrovska 33
**     624 00 Brno
**     Czech Republic
**     http      : www.processorexpert.com
**     mail      : info@processorexpert.com
** ###################################################################*/
/* MODULE DC17 */


/* Including needed modules to compile this module/procedure */
#include "Cpu.h"
#include "Events.h"
#include "MICOUT.h"
#include "MEM1.h"
#include "Term1.h"
#include "Inhr6.h"
#include "TI1.h"
#include "RedLED.h"
#include "GreenLED.h"
#include "BlueLED.h"
#include "AudioOutputPWM.h"
#include "PORTC6IN.h"
#include "PORTCOUT.h"
#include "PORTA4OUT.h"
#include "KeypadColumnsIn.h"
#include "KeypadRowOut0.h"
#include "KeypadRowOut1.h"
#include "KeypadRowOut2.h"
#include "KeypadRowOut3.h"
#include "MICPWR.h"
#include "DFR1.h"
/* Including shared modules, which are used for whole project */
#include "PE_Types.h"
#include "PE_Error.h"
#include "PE_Const.h"
#include "IO_Map.h"

#include "Keypad.h"
#include "Redbox.h"
#include "ToneGenerate.h"

typedef enum
{
	BADGE_BLUEBOX_MODE,
	BADGE_DTMF_MODE,
	BADGE_REDBOX_MODE,
	BADGE_VOICE_MODE,
	BADGE_MAX_MODES
}	BadgeOperatingMode_t;


#define MAIN_LOOP_SCAN_FREQUENCY		8000
#define KEYPAD_SCAN_FREQUENCY				125
#define HEARTBEAT_TOGGLE_FREQUENCY	2

#define KEYPAD_SCAN_INTERVAL			(MAIN_LOOP_SCAN_FREQUENCY / KEYPAD_SCAN_FREQUENCY)
#define HEARTBEAT_TOGGLE_INTERVAL	(MAIN_LOOP_SCAN_FREQUENCY / HEARTBEAT_TOGGLE_FREQUENCY)

static const char StartupString[] = "DC17 Badge (hacked by 501d3r guy) - Blue box/DTMF dialer/Red box (rev 01/07/2011)\r\n";

void main(void)
{
	UWord16 KeypadScanCode = KEYPAD_NOKEY;
	UWord16 PriorKeypadScanCode = KEYPAD_NOKEY;
	UWord16 HeartbeatCounter = 0;
	UWord16 KeypadScanCounter = 0;
	UWord8 HeartbeatToggleCounter = 0;
	Keypress_t CurrentKeyPressed = KEY_PRESS_INVALID;
	BadgeOperatingMode_t CurrentBadgeMode = BADGE_BLUEBOX_MODE;
	unsigned int MicrophoneOutput;
	byte Status;

  /*** Processor Expert internal initialization. DON'T REMOVE THIS CODE!!! ***/
  PE_low_level_init();
  /*** End of Processor Expert internal initialization.                    ***/

	InitializeKeypad();
	AudioOutputPWM_Enable();
	AudioOutputPWM_SetRatio16(32768);
	// workaround for generic PWM or CPU bean problem - cannot configure to allow
	// PWM to run while in WAIT mode
	{
		UWord16 PWMConfig;
		
		PWMConfig = getReg(PWM_CNFG);
		PWMConfig |= (1 << 13);
		setReg(PWM_CNFG, PWMConfig);
	}
	
	// output startup message via serial port
	Term1_SendStr((void *)StartupString);

  /* Write your code here */
	for(;;)
	{
		// update tone generation (if needed) based on current badge mode
		if(BADGE_BLUEBOX_MODE == CurrentBadgeMode)
			PlayTone(CurrentKeyPressed, KEYTONE_MODE_MF, MAIN_LOOP_SCAN_FREQUENCY);
		else if(BADGE_DTMF_MODE == CurrentBadgeMode)
			PlayTone(CurrentKeyPressed, KEYTONE_MODE_DTMF, MAIN_LOOP_SCAN_FREQUENCY);
		else if(BADGE_REDBOX_MODE == CurrentBadgeMode)
			RunRedboxFSM(CurrentKeyPressed, MAIN_LOOP_SCAN_FREQUENCY);
		else if(BADGE_VOICE_MODE == CurrentBadgeMode)
		{
			// directly pipe microphone input to speaker output for now
			Status =  MICOUT_Measure(TRUE);
			Status = MICOUT_GetValue16(&MicrophoneOutput);
			
			AudioOutputPWM_SetRatio16(MicrophoneOutput);
		}
	
		// update keypad scan timer, scan keypad if it is time
		KeypadScanCounter++;
		if(KeypadScanCounter >= KEYPAD_SCAN_INTERVAL)
		{
			KeypadScanCounter = 0;

			KeypadScanCode = ScanKeypad();

			// map keypad scan code to keypress value
			// precondition CurrentKeyPressed to "invalid"
			CurrentKeyPressed = KEY_PRESS_INVALID;

			if(KEYPAD_1 == KeypadScanCode)
			{
				CurrentKeyPressed = KEY_PRESS_1;
			}
			else if(KEYPAD_2 == KeypadScanCode)
			{
				CurrentKeyPressed = KEY_PRESS_2;
			}
			else if(KEYPAD_3 == KeypadScanCode)
			{
				CurrentKeyPressed = KEY_PRESS_3;
			}
			else if(KEYPAD_4 == KeypadScanCode)
			{
				CurrentKeyPressed = KEY_PRESS_4;
			}
			else if(KEYPAD_5 == KeypadScanCode)
			{
				CurrentKeyPressed = KEY_PRESS_5;
			}
			else if(KEYPAD_6 == KeypadScanCode)
			{
				CurrentKeyPressed = KEY_PRESS_6;
			}
			else if(KEYPAD_7 == KeypadScanCode)
			{
				CurrentKeyPressed = KEY_PRESS_7;
			}
			else if(KEYPAD_8 == KeypadScanCode)
			{
				CurrentKeyPressed = KEY_PRESS_8;
			}
			else if(KEYPAD_9 == KeypadScanCode)
			{
				CurrentKeyPressed = KEY_PRESS_9;
			}
			else if(KEYPAD_0 == KeypadScanCode)
			{
				CurrentKeyPressed = KEY_PRESS_0;
			}
			else if(KEYPAD_STAR == KeypadScanCode)
			{
				CurrentKeyPressed = KEY_PRESS_STAR;
			}
			else if(KEYPAD_POUND == KeypadScanCode)
			{
				CurrentKeyPressed = KEY_PRESS_POUND;
			}
			else if(KEYPAD_A == KeypadScanCode)
			{
				CurrentKeyPressed = KEY_PRESS_A;
			}
			else if(KEYPAD_B == KeypadScanCode)
			{
				CurrentKeyPressed = KEY_PRESS_B;
			}
			else if(KEYPAD_C == KeypadScanCode)
			{
				CurrentKeyPressed = KEY_PRESS_C;
			}
			else if(KEYPAD_D == KeypadScanCode)
			{
				CurrentKeyPressed = KEY_PRESS_D;
			}
			else if(KEYPAD_STAR_1 == KeypadScanCode)
			{
				// switch to bluebox mode
				CurrentBadgeMode = BADGE_BLUEBOX_MODE;
			}
			else if(KEYPAD_STAR_2 == KeypadScanCode)
			{
				// switch to DTMF mode
				CurrentBadgeMode = BADGE_DTMF_MODE;
			}
			else if(KEYPAD_STAR_3 == KeypadScanCode)
			{
				// switch to redbox mode
				CurrentBadgeMode = BADGE_REDBOX_MODE;
			}
			else if(KEYPAD_STAR_4 == KeypadScanCode)
			{
				// switch to voice mode
				CurrentBadgeMode = BADGE_VOICE_MODE;
			}
			
			PriorKeypadScanCode = KeypadScanCode;
		}
	
		// update heartbeat/mode toggle timer, toggle appropriate LED if it is time
		HeartbeatCounter++;
		if(HeartbeatCounter >= HEARTBEAT_TOGGLE_INTERVAL)
		{
			HeartbeatCounter = 0;
			
			HeartbeatToggleCounter++;
			
			if(HeartbeatToggleCounter >= 2)
			{
				HeartbeatToggleCounter = 0;
			}
			
			if(1 == HeartbeatToggleCounter)
			{
				if(BADGE_BLUEBOX_MODE == CurrentBadgeMode)
				{
					// bluebox mode - turn on blue LED, turn all others off
					RedLED_SetVal();
					GreenLED_SetVal();
					BlueLED_ClrVal();
				}
				else if(BADGE_REDBOX_MODE == CurrentBadgeMode)
				{
					// redbox mode - turn on red LED, turn all others off
					RedLED_ClrVal();
					GreenLED_SetVal();
					BlueLED_SetVal();
				}
				else if(BADGE_VOICE_MODE == CurrentBadgeMode)
				{
					// voice mode - turn all LEDS off (for now until we can blend colors)
					RedLED_SetVal();
					GreenLED_SetVal();
					BlueLED_SetVal();
				}
				else
				{
					// DTMF mode - turn on green LED, turn all others off
					RedLED_SetVal();
					GreenLED_ClrVal();
					BlueLED_SetVal();
				}
			}
			else
			{
				// all LEDs off
				RedLED_SetVal();
				GreenLED_SetVal();
				BlueLED_SetVal();
			}
		}
	
		// run in wait mode until the next interrupt
		Cpu_SetWaitMode();
	}
}


/*
** ###################################################################
**
**     This file was created by UNIS Processor Expert 2.99 [04.17]
**     for the Freescale 56800 series of microcontrollers.
**
** ###################################################################
*/
